/*
** folder_read.c -- initialize folder structure and read folder
**
** This code is Copyright (c) 2002, by the authors of nmh.  See the
** COPYRIGHT file in the root directory of the nmh distribution for
** complete copyright information.
*/

#include <unistd.h>
#include <h/mh.h>
#include <h/utils.h>
#include <dirent.h>
#include <sys/stat.h>

/*
** 1) Create the folder/message structure
** 2) Read the directory (folder) and temporarily
**    record the numbers of the messages we have seen.
** 3) Then allocate the array for message attributes and
**    set the initial flags for all messages we've seen.
** 4) Read and initialize the sequence information.
*/

static int others;

static int
msgnumcmp(const struct dirent **d1, const struct dirent **d2)
{
	size_t l1, l2;

	l1 = strlen((*d1)->d_name);
	l2 = strlen((*d2)->d_name);
	if (l1 < l2) {
		return -1;
	}
	if (l1 > l2) {
		return 1;
	}
	return strcmp((*d1)->d_name, (*d2)->d_name);
}

static int
msgfilter(const struct dirent *e)
{
	int i;
	/* For compatibility with nmh, ignore rmm backup files. */
	if (e->d_name[0] == '.' || e->d_name[0] == ',' || e->d_name[0] == '#') {
		return 0;
	}
	for (i = 0; e->d_name[i]; i++) {
		if ((i == 0 && e->d_name[i] == '0') || e->d_name[i] < '0' || e->d_name[i] > '9') {
			others = 1;
			return 0;
		}
	}
	return 1;
}

struct msgs *
folder_read(char *name)
{
	int i;
	struct msgs *mp;
	struct stat st;
	struct dirent **dp;

	others = 0;
	name = mh_xstrdup(toabsdir(name));

	if (stat(name, &st) == -1) {
		mh_free0(&name);
		return NULL;
	}

	/* Allocate the main structure for folder information */
	mp = mh_xcalloc(1, sizeof(*mp));

	mp->foldpath = name;

	if (access(name, W_OK) == -1)
		set_readonly(mp);

	mp->nummsg = scandir(name, &dp, msgfilter, msgnumcmp);

	if (others)
		set_other_files(mp);

	if (mp->nummsg < 0) {
		mh_free0(&name);
		mh_free0(&mp);
		return NULL;
	}
	if (mp->nummsg > 0) {
		mp->lowmsg = m_atoi(dp[0]->d_name);
		mp->hghmsg = m_atoi(dp[mp->nummsg-1]->d_name);
	}
	mp->lowoff = max(mp->lowmsg, 1);
	mp->hghoff = mp->hghmsg + 100;
	mp->msgstats = mh_xcalloc(MSGSTATSIZE(mp, mp->lowoff, mp->hghoff), 1);
	for (i = 0; i < mp->nummsg; i++) {
		set_exists(mp, m_atoi(dp[i]->d_name));
		mh_free0(dp + i);
	}
	mh_free0(&dp);

	/*
	** Read and initialize the sequence information.
	*/
	seq_read(mp);

	return mp;
}
